-- Code de l'environnement de test.
-- Ce module effectue un test exhaustif des 2  versions de l'additionneur 4 bit :
--      - structurel
--      - comportemental

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity test is
end entity;

architecture arc of test is
--Declaration des signaux (equipotentielles, noeuds) internes.
-- entres
  signal a, b : unsigned(3 downto 0);

-- sorties
  signal s1, s2       : unsigned (3 downto 0);
  signal cout1, cout2 : std_logic;

-- Autres noeuds internes
  signal i, j, k      : integer := 0; 
  signal error_found1 : integer := 0;  -- erreur sur le modle structurel. 
  signal error_found2 : integer := 0;  -- erreur sur le modle comportemental. 
  signal clk          : std_logic;  --     : on synchronise les tests sur une horloge. C'est pratique au cas ou les additionneurs ont un temps de propagation : on prend alors une horloge suffisament lente pour leur laisser le temps de se stabiliser. On ne teste leur sortie qu'au front d'horloge suivant.

  component add_struct is
    port
      (
        a, b : in  unsigned(3 downto 0);
        s    : out unsigned(3 downto 0);
        cout : out std_logic
        );
  end component;

  component add_comp is
    port (
      a, b : in  unsigned(3 downto 0);
      s    : out unsigned(3 downto 0);
      cout : out std_logic
      );
  end component;

begin

-- Instanciation des additionneurs. 
  inst1 : add_struct port map (a, b, s1, cout1);
  inst2 : add_comp port map (a, b, s2, cout2);

-- Generation des vecteurs de test
--   On commence par initialiser tous les signaux a zero, et afficher un message de bienvenue


-- on genere une horloge de 10ns de periode
  process
  begin
    clk <= '0';
    wait for 5 ns;
    clk <= '1';
    wait for 5 ns;
  end process;

-- A chaque front montant de l'horloge, on commence par verifier que les sorties de additionneurs sont correctes.
-- Puis on change de combinaison de test
-- Si on a teste toutes les combinaisons, on s'arrete

  a <= to_unsigned (i, 4);
  b <= to_unsigned (j, 4);
  k <= i+j;

  process
  begin
    -- Test des sorties
    wait until clk'event and clk = '1';
    if k /= to_integer(cout1 & s1) then
      error_found1 <= 1;
    end if;
    if k /= to_integer(cout2 & s2) then
      error_found2 <= 1;
    end if;

    if i = 15 then
      i <= 0;
      j <= j + 1;
    else
      i <= i + 1;
    end if;

    assert (not(j = 15 and i = 15)) report "fin de simulation" severity failure;
    assert (error_found1 = 0) report "erreur sur l'additionneur structurel" severity error;
    assert (error_found2 = 0) report "erreur sur l'additionneur comportemental" severity error;
    
  end process;




end arc;
